
BEGIN {
    unless ( $ENV{AUTHOR_TESTING} ) {
        require Test::More;
        Test::More::plan(
            skip_all => 'these tests are for testing by the author' );
    }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.14

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/Test/Mocha.pm',                  'lib/Test/Mocha/CalledOk.pm',
    'lib/Test/Mocha/CalledOk/AtLeast.pm', 'lib/Test/Mocha/CalledOk/AtMost.pm',
    'lib/Test/Mocha/CalledOk/Between.pm', 'lib/Test/Mocha/CalledOk/Times.pm',
    'lib/Test/Mocha/Inspect.pm',          'lib/Test/Mocha/Method.pm',
    'lib/Test/Mocha/MethodCall.pm',       'lib/Test/Mocha/MethodStub.pm',
    'lib/Test/Mocha/Mock.pm',             'lib/Test/Mocha/PartialDump.pm',
    'lib/Test/Mocha/Stub.pm',             'lib/Test/Mocha/Types.pm',
    'lib/Test/Mocha/Util.pm',             'lib/Test/Mocha/Verify.pm',
    't/author-critic.t',                  't/author-no-tabs.t',
    't/called_ok.t',                      't/class_mock.t',
    't/clear.t',                          't/inspect.t',
    't/inspect_all.t',                    't/matcher_moose.t',
    't/matcher_typetiny.t',               't/mock.t',
    't/partialdump.t',                    't/release-pod-coverage.t',
    't/release-pod-syntax.t',             't/smartmatch.t',
    't/stub.t',                           't/universal.t',
    't/v0.21/matcher_moose.t',            't/v0.21/matcher_typetiny.t',
    't/v0.21/universal.t',                't/v0.21/verify.t',
    't/v0.50/called_ok.t',                't/v0.50/clear.t',
    't/v0.50/inspect.t',                  't/v0.50/inspect_all.t',
    't/v0.50/matcher_moose.t',            't/v0.50/matcher_typetiny.t',
    't/v0.50/smartmatch.t',               't/v0.50/stub.t',
    't/v0.50/universal.t'
);

notabs_ok($_) foreach @files;
done_testing;
